verify_lock(): report errors via a strbuf
[gitweb.git] / refs.c
diff --git a/refs.c b/refs.c
index 27b769afddb97bd9ab6611cf16761666093ae23c..3a73cf2f164dcc71a74e59ce7f599d1631b78c12 100644 (file)
--- a/refs.c
+++ b/refs.c
@@ -2221,25 +2221,28 @@ static void unlock_ref(struct ref_lock *lock)
 /*
  * Verify that the reference locked by lock has the value old_sha1.
  * Fail if the reference doesn't exist and mustexist is set. Return 0
* on success or a negative value on error. This function should make
* sure errno is meaningful on error.
  */
 static int verify_lock(struct ref_lock *lock,
-                      const unsigned char *old_sha1, int mustexist)
+                      const unsigned char *old_sha1, int mustexist,
+                      struct strbuf *err)
 {
+       assert(err);
+
        if (read_ref_full(lock->ref_name,
                          mustexist ? RESOLVE_REF_READING : 0,
                          lock->old_sha1, NULL)) {
                int save_errno = errno;
-               error("Can't verify ref %s", lock->ref_name);
-               unlock_ref(lock);
+               strbuf_addf(err, "Can't verify ref %s", lock->ref_name);
                errno = save_errno;
                return -1;
        }
        if (hashcmp(lock->old_sha1, old_sha1)) {
-               error("Ref %s is at %s but expected %s", lock->ref_name,
-                       sha1_to_hex(lock->old_sha1), sha1_to_hex(old_sha1));
-               unlock_ref(lock);
+               strbuf_addf(err, "Ref %s is at %s but expected %s",
+                           lock->ref_name,
+                           sha1_to_hex(lock->old_sha1),
+                           sha1_to_hex(old_sha1));
                errno = EBUSY;
                return -1;
        }
@@ -2471,8 +2474,10 @@ static struct ref_lock *lock_ref_sha1_basic(const char *refname,
                        goto error_return;
                }
        }
-       if (old_sha1 && verify_lock(lock, old_sha1, mustexist))
-               return NULL;
+       if (old_sha1 && verify_lock(lock, old_sha1, mustexist, err)) {
+               last_errno = errno;
+               goto error_return;
+       }
        return lock;
 
  error_return:
@@ -2221,25 +2221,28 @@ static void unlock_ref(struct ref_lock *lock)
  * Verify that the reference locked by lock has the value old_sha1.
  * Fail if the reference doesn't exist and mustexist is set. Return 0
- * on success or a negative value on error. This function should make
- * sure errno is meaningful on error.
+ * on success. On error, write an error message to err, set errno, and
+ * return a negative value.
  */
 static int verify_lock(struct ref_lock *lock,